% Copyright (c) 2020 
% Authors:
% F. Pierre, M. Amendola, C. Bigeard, T. Ruel
% and P.-F. Villard
% 
% This program is free software: you can redistribute it and/or 
% modify it under the terms of the GNU Affero General Public 
% License as published by the Free Software Foundation, either 
% version 3 of the License, or (at your option) any later 
% version.
% 
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied 
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
% PURPOSE. See the GNU Affero General Public License for more 
% details.
% 
% You should have received a copy of the GNU Affero General 
% Public License along with this program. If not, 
% see <http://www.gnu.org/licenses/>.


function  A = computeA(n,alpha,beta)
% A = computeA(n, alpha, beta)
% Computes the circular matrix of size n containing on each line
% $[\beta, -\alpha-4\beta, 6\beta+2\alpha, -\alpha-4\beta, \beta]$. 

% Construction of the diagonal.
d1=-2*ones(1,n);

% Construction of sub/sur-diagonal.
d2=ones(1,n-1);

% Matrix assembly.
A2=diag(d1) + diag(d2,1) + diag(d2,-1);% Square of Equation (14) is an 
                              % approximation of Equation (15) in the paper

% Boundary conditions.
A2(1,n)=1;
A2(n,1)=1;

% Derivative operator order 4.
A4=A2^2;% Equation (15) in the paper


% Final matrix. 
A=-alpha*A2+beta*A4;
