% Copyright (c) 2020 
% Authors:
% F. Pierre, M. Amendola, C. Bigeard, T. Ruel
% and P.-F. Villard
% 
% This program is free software: you can redistribute it and/or 
% modify it under the terms of the GNU Affero General Public 
% License as published by the Free Software Foundation, either 
% version 3 of the License, or (at your option) any later 
% version.
% 
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied 
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
% PURPOSE. See the GNU Affero General Public License for more 
% details.
% 
% You should have received a copy of the GNU Affero General 
% Public License along with this program. If not, 
% see <http://www.gnu.org/licenses/>.

function parity = polygonParity(V)
% Let V be a polygon.  polygonParity computes an integer which value is 
% equal to 1 if the interior of the polygon is right-handed when the points
% are browsed in the regular order. 
% -1 if the interior of the polygon is left-handed when the points are
% browsed in the regular order. 

vn1 = [V(end,1)-V(1,1), V(end,2)-V(1,2)] /...
            (sqrt((V(end,1)-V(1,1))^2+(V(end,2)-V(1,2))^2));
vn2 = [V(1,1)-V(2,1), V(1,2)-V(2,2)] /...
            (sqrt((V(1,1)-V(2,1))^2+(V(1,2)-V(2,2))^2));
vb = [vn1(1) + vn2(1), vn1(2) + vn2(2)];

a_eb = vb(1);
b_eb = vb(2);
c_eb = vb(1) * V(1,1) + vb(2) * V(1,2);

c=0;

for i = 2:size(V,1)-1
    denom = a_eb * V(i,1) - a_eb * V(i+1,1) +...
                            b_eb * V(i,2) - b_eb * V(i+1,2);
    if denom ~= 0
        t = (c_eb - a_eb * V(i+1,1) - b_eb * V(i+1,2)) / denom;
        if t >= 0 && t < 1
            vt = [t * V(i,1) + (1-t) * V(i+1,1) - V(1,1),...
                            t * V(i,2) + (1-t) * V(i+1,2) - V(1,2)];
            nt = [vt(2), -vt(1)];
            scal = nt * vb';
            if scal > 0
                c = c + 1;
            end
        end
    end
end

if mod(c,2) == 0
    parity = 1;
else
    parity = -1;
end

end

