% Copyright (c) 2020 
% Authors:
% F. Pierre, M. Amendola, C. Bigeard, T. Ruel
% and P.-F. Villard
% 
% This program is free software: you can redistribute it and/or 
% modify it under the terms of the GNU Affero General Public 
% License as published by the Free Software Foundation, either 
% version 3 of the License, or (at your option) any later 
% version.
% 
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied 
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
% PURPOSE. See the GNU Affero General Public License for more 
% details.
% 
% You should have received a copy of the GNU Affero General 
% Public License along with this program. If not, 
% see <http://www.gnu.org/licenses/>.

function fc = smoothForces(f, sig)
% returns a vector of force f averaged with its neighbours in a circular manner 
% in order to take into account all the influences of the neighbours in the 
% spline, which is a closed contour:
% We use (n+1)/2 on the right of the points and (n+1)/2 points on its
% left.

% smoothForces performs a circular convolution of the force vector f.
% sig = parameter of the exponential decay
%
% It corresponds to equation (27) in the article

n=length(f(:,1));
l2=max(1,floor(n/2));
t=[0:l2-1 , -(n-l2):-1];
P=exp(-t.^2/(abs(sig)+1e-6));
P=P/sum(P(:));
fc1 = real(ifft(fft(f(:,1)).*fft(P(:)))); 
fc2 = real(ifft(fft(f(:,2)).*fft(P(:)))); 
fc = [fc1 fc2];

