% Copyright (c) 2020 
% Authors:
% F. Pierre, M. Amendola, C. Bigeard, T. Ruel
% and P.-F. Villard
% 
% This program is free software: you can redistribute it and/or 
% modify it under the terms of the GNU Affero General Public 
% License as published by the Free Software Foundation, either 
% version 3 of the License, or (at your option) any later 
% version.
% 
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied 
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
% PURPOSE. See the GNU Affero General Public License for more 
% details.
% 
% You should have received a copy of the GNU Affero General 
% Public License along with this program. If not, 
% see <http://www.gnu.org/licenses/>.

function PT = splinesInterpolation2D( V, NbPoints )
%splinesInterpolation2D : interpolation with cubic splines
% Interpolates closed path V and samples NbPoints regularly
% distributed

N2 = NbPoints;  

N = size(V,1); 

W = spdiags([ones(N,1),-2*ones(N,1),ones(N,1)],[-1,0,1], N,N);
W(1,N) = 1; W(N,1) = 1; 

VV = (W*V).'; 
d = N^2*VV; 
d=d';

M = spdiags([2/3 * ones(N, 1), 1/6*ones(N, 1), 1/6*ones(N, 1), ...
    1/6*ones(N, 1), 1/6*ones(N, 1)],...
    [0, 1, -1, N - 1, -(N - 1)], N, N); 

x = M\d(:,1); 
y = M\d(:,2); 
X = [x,y];

L = zeros(N,1); 
NT = 100*N; 

% Computation of equation (37) by a left Riemann sum (38)
for i=1:N
    A = linspace(0, 1 ,100)'; 
    B = 1-A; 
    der  =  N*(V(1+mod(i,N),:) - V(i,:))  ...
          + (-(3*A.*A-1)* X(i,:) + (3*B.*B-1)* X(1+mod(i,N),:))/(6*N); 
    L(i) = sum(sqrt(sum(der.*der,2))/NT); 
end

PT = zeros(N2, 2); 
a = linspace(0, 1, N2)'; 
L = cumsum(L); 
L = [0; L/L(end)]; 

for i=1:N
    condi = (a >= L(i) & a < L(i+1)) ...
        | (i == N & a >= L(N)) | (i == 1 & a < L(i)); 
    A = (a(condi) - L(i + 1)) / (L(i) - L(i + 1)); 
    B = 1 - A; 
    C = 1/6 * (A.^3 - A) * (i/N - (i-1)/N)^2; 
    D = 1/6 * (B.^3 - B) * (i/N - (i-1)/N)^2; 
    % Sampling Equation (32) in the paper:
    xy = A*V(i,:) + B*V(1+mod(i,N),:) + C*X(i,:) + D*X(1+mod(i,N),:); 
    PT(condi,:) = xy; 
end

end
