% Copyright (c) 2020 
% Authors:
% F. Pierre, M. Amendola, C. Bigeard, T. Ruel
% and P.-F. Villard
% 
% This program is free software: you can redistribute it and/or 
% modify it under the terms of the GNU Affero General Public 
% License as published by the Free Software Foundation, either 
% version 3 of the License, or (at your option) any later 
% version.
% 
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied 
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
% PURPOSE. See the GNU Affero General Public License for more 
% details.
% 
% You should have received a copy of the GNU Affero General 
% Public License along with this program. If not, 
% see <http://www.gnu.org/licenses/>.

%Compute 3D normals
function N = norm3D(face,vertex)
%Vertex normals of a triangulated mesh, area weighted, left-hand-rule
% N = patchnormals(FV) -struct with fields, faces Nx3 and vertices Mx3
%N: vertex normals as Mx3

%face corners index
A = face(:,1);
B = face(:,2);
C = face(:,3);

%face normals
n = cross(vertex(A,:)-vertex(B,:),vertex(C,:)-vertex(A,:)); %area weighted
% Equation (48) in the paper


%vertice normals
N = zeros(size(vertex)); %init vertix normals
for i = 1:size(face,1) %step through faces (a vertex can be reference any number of times)
N(A(i),:) = N(A(i),:)+n(i,:); %sum face normals
N(B(i),:) = N(B(i),:)+n(i,:);
N(C(i),:) = N(C(i),:)+n(i,:);
end


    
N=N./(repmat(sqrt(sum(N.^2,2)), [1,size(N,2)]));