% Copyright (c) 2020 
% Authors:
% F. Pierre, M. Amendola, C. Bigeard, T. Ruel
% and P.-F. Villard
% 
% This program is free software: you can redistribute it and/or 
% modify it under the terms of the GNU Affero General Public 
% License as published by the Free Software Foundation, either 
% version 3 of the License, or (at your option) any later 
% version.
% 
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied 
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
% PURPOSE. See the GNU Affero General Public License for more 
% details.
% 
% You should have received a copy of the GNU Affero General 
% Public License along with this program. If not, 
% see <http://www.gnu.org/licenses/>.

%Return adjacence matrix of a given mesh
function A = triangulation2adjacency(face)

% triangulation2adjacency - compute the adjacency matrix
%   of a given triangulation.
%
%   A = triangulation2adjacency(face);
% or for getting a weighted graph
%   A = triangulation2adjacency(face,vertex);
%
%   Copyright (c) 2005 Gabriel Peyr�


[~,face] = checkFaceVertex([],face);
f = double(face)';

A = sparse([f(:,1); f(:,1); f(:,2); f(:,2); f(:,3); f(:,3)], ...
           [f(:,2); f(:,3); f(:,1); f(:,3); f(:,1); f(:,2)], ...
           1.0);
% avoid double links
A = double(A>0);
end