% Copyright (c) 2020 
% Authors:
% F. Pierre, M. Amendola, C. Bigeard, T. Ruel
% and P.-F. Villard
% 
% This program is free software: you can redistribute it and/or 
% modify it under the terms of the GNU Affero General Public 
% License as published by the Free Software Foundation, either 
% version 3 of the License, or (at your option) any later 
% version.
% 
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the implied 
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
% PURPOSE. See the GNU Affero General Public License for more 
% details.
% 
% You should have received a copy of the GNU Affero General 
% Public License along with this program. If not, 
% see <http://www.gnu.org/licenses/>.

%Create inital mesh from a sphere
function [vertex, face]=trisphere(iterations, rayon, x_0, y_0, z_0)

    %face : (4^iterations) * 8

    p(1).x=0; p(1).y=0; p(1).z=1;
    p(2).x=0; p(2).y=0; p(2).z=-1;
    p(3).x=-1; p(3).y=-1; p(3).z=0;
    p(4).x=1; p(4).y=-1; p(4).z=0;
    p(5).x=1; p(5).y=1; p(5).z=0;
    p(6).x=-1; p(6).y=1; p(6).z=0;
    %Create the level 0 object 
    a = 1 / sqrt(2.0);
    for i=1:6 
      p(i).x = p(i).x*a;
      p(i).y = p(i).y*a;
    end
    f(1).p1 = p(1); f(1).p2 = p(4); f(1).p3 = p(5);
    f(2).p1 = p(1); f(2).p2 = p(5); f(2).p3 = p(6);
    f(3).p1 = p(1); f(3).p2 = p(6); f(3).p3 = p(3);
    f(4).p1 = p(1); f(4).p2 = p(3); f(4).p3 = p(4);
    f(5).p1 = p(2); f(5).p2 = p(5); f(5).p3 = p(4);
    f(6).p1 = p(2); f(6).p2 = p(6); f(6).p3 = p(5);
    f(7).p1 = p(2); f(7).p2 = p(3); f(7).p3 = p(6);
    f(8).p1 = p(2); f(8).p2 = p(4); f(8).p3 = p(3);  
    
    nt = 8;
    
    if (iterations < 1)
      return;
    end
    
    % Bisect each edge and move to the surface of a unit sphere 
    for it=1:iterations
      ntold = nt;
      for i=1:ntold
         pa.x = (f(i).p1.x + f(i).p2.x) / 2.;
         pa.y = (f(i).p1.y + f(i).p2.y) / 2.;
         pa.z = (f(i).p1.z + f(i).p2.z) / 2.;
         pb.x = (f(i).p2.x + f(i).p3.x) / 2.;
         pb.y = (f(i).p2.y + f(i).p3.y) / 2.;
         pb.z = (f(i).p2.z + f(i).p3.z) / 2.;
         pc.x = (f(i).p3.x + f(i).p1.x) / 2.;
         pc.y = (f(i).p3.y + f(i).p1.y) / 2.;
         pc.z = (f(i).p3.z + f(i).p1.z) / 2.;
         pa=normalise(pa);
         pb=normalise(pb);
         pc=normalise(pc);
         f(nt+1).p1 = f(i).p1; f(nt+1).p2 = pa; f(nt+1).p3 = pc; nt=nt+1;
         f(nt+1).p1 = pa; f(nt+1).p2 = f(i).p2; f(nt+1).p3 = pb; nt=nt+1;
         f(nt+1).p1 = pb; f(nt+1).p2 = f(i).p3; f(nt+1).p3 = pc; nt=nt+1;
         f(i).p1 = pa;
         f(i).p2 = pb;
         f(i).p3 = pc;
      end
    end
    close all;
    
    Pt1=[f(:).p1];
    Pt2=[f(:).p2];
    Pt3=[f(:).p3];
    allVertexPt1=[Pt1.x;Pt1.y;Pt1.z];
    allVertexPt2=[Pt2.x;Pt2.y;Pt2.z];
    allVertexPt3=[Pt3.x;Pt3.y;Pt3.z];
    allVertex=[ allVertexPt1';...
                allVertexPt2';...
                allVertexPt3'];
    vertex=unique(allVertex,'rows');
    [~, index]=ismember(allVertex,vertex,'rows');

    %face 1
    F1Pt1=1:3:nt;
    F1Pt2=nt+1:3:2*nt;
    F1Pt3=2*nt+1:3:3*nt;
    %face 2
    F2Pt1=2:3:nt+1;
    F2Pt2=nt+2:3:2*nt+1;
    F2Pt3=2*nt+2:3:3*nt;
    %face 3
    F3Pt1=3:3:nt;
    F3Pt2=nt+3:3:2*nt;
    F3Pt3=2*nt+3:3:3*nt;
    
    face1=[index(F1Pt1),index(F1Pt2),index(F1Pt3)];
    face2=[index(F2Pt1),index(F2Pt2),index(F2Pt3)];
    face3=[index(F3Pt1),index(F3Pt2),index(F3Pt3)];
    face=[face1;face2;face3];
    
    
    vertex = rayon*vertex;
    vertex(:,1) = vertex(:,1) + x_0;
    vertex(:,2) = vertex(:,2) + y_0;
    vertex(:,3) = vertex(:,3) + z_0;

end

function N=normalise(p)
    norm=sqrt(p.x^2+p.y^2+p.z^2);
    N.x=p.x/norm;
    N.y=p.y/norm;
    N.z=p.z/norm;
end

